package com.contextspace.objectdb_example.dao;

import java.io.Serializable;
import java.util.List;

import javax.persistence.EntityManager;

import org.springframework.transaction.annotation.Transactional;

import com.googlecode.genericdao.dao.jpa.GenericDAO;

public interface IBaseDAO<T, ID extends Serializable> extends GenericDAO<T, Long> {
	/**
	 * Persists a new entity
	 * @param entityToCreate
	 * @return The newly persisted entity
	 */
	@Transactional
	public T create(T entityToCreate);
	
	/**
	 * Persists a list of new entities
	 * @param entities
	 * @return A List of the newley persisted entities
	 */
	@Transactional
	public List<T> create(T... entities);
	
	/**
	 * This has been added due to an issue we are having with find. Seems find is getting knotted over nested maps, and
	 * not populating them on return. By doing a full select everything seems to get populated as expected
	 * @param id
	 * @return
	 * @throws Exception
	 */
	public T lookupById(long id) throws Exception;
	
	/**
	 * Persists changes to an existing entity
	 * @param entityToUpdate
	 * @return The updated entity
	 */
	@Transactional
	public T update(T entityToUpdate);
	
	/**
	 * Persists changes to a list of existing entities
	 * @param entitiesToUpdate
	 * @return The list of all updated entities
	 */
	@Transactional
	public List<T> update(T... entitiesToUpdate);
	
	/**
	 * Deletes an existing entity
	 * @param entityToDelete
	 * @return true if the entity was deleted successfully or false otherwise
	 */
	@Transactional
	public boolean delete(T entityToDelete);
	
	/**
	 * Deletes a list of existing entities
	 * @param entitiesToDelete
	 * @return true if all entities in entitiesToDelete were deleted successfully or false otherwise
	 */
	@Transactional
	public boolean delete(T... entitiesToDelete);
	
	/**
	 * Deletes a list of existing entities
	 * @param entitiesToDelete
	 * @return true if all entities in entitiesToDelete were deleted successfully or false otherwise
	 */
	@Transactional
	public boolean delete(List<T> entitiesToDelete);
	
	/**
	 * Deletes an existing entity of type <T> identified by <ID> id 
	 * @param id
	 * @return true if the entity was deleted successfully or false otherwise
	 */
	@Transactional
	public boolean deleteByID(long id);
	
	/**
	 * detaches the entity from the entity manager
	 * @param entity
	 */
	public void detach(T entity);
	   
    /**
     * @return
     */
    public EntityManager getEntityManager();
}
