package com.contextspace;

import static org.junit.Assert.*;

import java.io.File;
import java.util.List;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;
import org.springframework.context.support.ClassPathXmlApplicationContext;

import com.contextspace.objectdb_example.dao.IRuleDAO;
import com.contextspace.objectdb_example.model.Rule;
import com.contextspace.objectdb_example.model.RulePart;

public class ExampleTest {

    private static ClassPathXmlApplicationContext context = null;
    
    private static IRuleDAO ruleDao;
    
    @BeforeClass
    public static void setUpBeforeClass() throws Exception {
        context = new ClassPathXmlApplicationContext(new String[] { "applicationContext-test.xml" });
     
        ruleDao = context.getBean(IRuleDAO.class);
    }
    
    @AfterClass
    public static void tearDownAfterClass() throws Exception {
        if(context != null) {
            context.close();
        }      
        
        //File f1 = new File("objectdb"+File.separator+"db");
        //deleteDir(f1);
    }
    
    
    @Test
    public void test() throws Exception {
        
        Rule newRule = new Rule("Test rule");
        
        RulePart part1 = new RulePart();
        part1.setAttributeName("attr1");
        part1.setOperation("equals");
        part1.setValue("value");
        
        RulePart part2 = new RulePart();
        part2.setAttributeName("attr2");
        part2.setOperation("<");
        part2.setValue("1");
        
        newRule.addRulePart(part1);
        newRule.addRulePart(part2);
        
        newRule.setCompiledRule("Some compiled rule");
        
        newRule = ruleDao.create(newRule);
        ruleDao.flush();
        
        assertTrue(newRule.getId() > 0);
        
        List<Rule> ruleList = ruleDao.findAll();
        assertEquals(1, ruleList.size());
        
        //Rule retrieveRule = ruleDao.find(newRule.getId());
        Rule retrieveRule = ruleList.get(0);
        assertNotNull(retrieveRule.getRules());
        assertEquals(2, retrieveRule.getRules().size());
        assertEquals("Test rule", retrieveRule.getName());
        
        
    }
    
    private static boolean deleteDir(File dir) {
        if (dir.isDirectory()) {
            String[] children = dir.list();
            for (int i=0; i<children.length; i++) {
                boolean success = deleteDir(new File(dir, children[i]));
                if (!success) {
                    return false;
                }
            }
        }
    
        // The directory is now empty so delete it
        return dir.delete();
    } 
    
}
